// Code generated by sqlc. DO NOT EDIT.
// source: transactions.sql

package postgres

import (
	"context"
	"time"

	"git.javil.eu/jacob1123/budgeteer/postgres/numeric"
	"github.com/google/uuid"
)

const createTransaction = `-- name: CreateTransaction :one
INSERT INTO transactions
(date, memo, amount, account_id, payee_id, category_id, group_id, status)
VALUES ($1, $2, $3, $4, $5, $6, $7, $8)
RETURNING id
`

type CreateTransactionParams struct {
	Date       time.Time
	Memo       string
	Amount     numeric.Numeric
	AccountID  uuid.UUID
	PayeeID    uuid.NullUUID
	CategoryID uuid.NullUUID
	GroupID    uuid.NullUUID
	Status     TransactionStatus
}

func (q *Queries) CreateTransaction(ctx context.Context, arg CreateTransactionParams) (uuid.UUID, error) {
	row := q.db.QueryRowContext(ctx, createTransaction,
		arg.Date,
		arg.Memo,
		arg.Amount,
		arg.AccountID,
		arg.PayeeID,
		arg.CategoryID,
		arg.GroupID,
		arg.Status,
	)
	var id uuid.UUID
	err := row.Scan(&id)
	return id, err
}

const deleteAllTransactions = `-- name: DeleteAllTransactions :execrows
DELETE FROM transactions
USING accounts
WHERE accounts.budget_id = $1
AND accounts.id = transactions.account_id
`

func (q *Queries) DeleteAllTransactions(ctx context.Context, budgetID uuid.UUID) (int64, error) {
	result, err := q.db.ExecContext(ctx, deleteAllTransactions, budgetID)
	if err != nil {
		return 0, err
	}
	return result.RowsAffected()
}

const deleteTransaction = `-- name: DeleteTransaction :exec
DELETE FROM transactions
WHERE id = $1
`

func (q *Queries) DeleteTransaction(ctx context.Context, id uuid.UUID) error {
	_, err := q.db.ExecContext(ctx, deleteTransaction, id)
	return err
}

const getAllTransactionsForBudget = `-- name: GetAllTransactionsForBudget :many
SELECT  t.id, t.date, t.memo, t.amount, t.group_id, t.status, t.account, t.payee_id, t.category_id, t.payee, t.category_group, t.category, t.transfer_account, t.budget_id, t.account_id
FROM display_transactions AS t
WHERE t.budget_id = $1
`

func (q *Queries) GetAllTransactionsForBudget(ctx context.Context, budgetID uuid.UUID) ([]DisplayTransaction, error) {
	rows, err := q.db.QueryContext(ctx, getAllTransactionsForBudget, budgetID)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []DisplayTransaction
	for rows.Next() {
		var i DisplayTransaction
		if err := rows.Scan(
			&i.ID,
			&i.Date,
			&i.Memo,
			&i.Amount,
			&i.GroupID,
			&i.Status,
			&i.Account,
			&i.PayeeID,
			&i.CategoryID,
			&i.Payee,
			&i.CategoryGroup,
			&i.Category,
			&i.TransferAccount,
			&i.BudgetID,
			&i.AccountID,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const getTransaction = `-- name: GetTransaction :one
SELECT id, date, memo, amount, group_id, status, account, payee_id, category_id, payee, category_group, category, transfer_account, budget_id, account_id FROM display_transactions
WHERE id = $1
`

func (q *Queries) GetTransaction(ctx context.Context, id uuid.UUID) (DisplayTransaction, error) {
	row := q.db.QueryRowContext(ctx, getTransaction, id)
	var i DisplayTransaction
	err := row.Scan(
		&i.ID,
		&i.Date,
		&i.Memo,
		&i.Amount,
		&i.GroupID,
		&i.Status,
		&i.Account,
		&i.PayeeID,
		&i.CategoryID,
		&i.Payee,
		&i.CategoryGroup,
		&i.Category,
		&i.TransferAccount,
		&i.BudgetID,
		&i.AccountID,
	)
	return i, err
}

const getTransactionsByMonthAndCategory = `-- name: GetTransactionsByMonthAndCategory :many
SELECT date, category_id, budget_id, amount
FROM transactions_by_month
WHERE transactions_by_month.budget_id = $1
`

func (q *Queries) GetTransactionsByMonthAndCategory(ctx context.Context, budgetID uuid.UUID) ([]TransactionsByMonth, error) {
	rows, err := q.db.QueryContext(ctx, getTransactionsByMonthAndCategory, budgetID)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []TransactionsByMonth
	for rows.Next() {
		var i TransactionsByMonth
		if err := rows.Scan(
			&i.Date,
			&i.CategoryID,
			&i.BudgetID,
			&i.Amount,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const getTransactionsForAccount = `-- name: GetTransactionsForAccount :many
SELECT  t.id, t.date, t.memo, t.amount, t.group_id, t.status, t.account, t.payee_id, t.category_id, t.payee, t.category_group, t.category, t.transfer_account, t.budget_id, t.account_id
FROM display_transactions AS t
WHERE t.account_id = $1
LIMIT 200
`

func (q *Queries) GetTransactionsForAccount(ctx context.Context, accountID uuid.UUID) ([]DisplayTransaction, error) {
	rows, err := q.db.QueryContext(ctx, getTransactionsForAccount, accountID)
	if err != nil {
		return nil, err
	}
	defer rows.Close()
	var items []DisplayTransaction
	for rows.Next() {
		var i DisplayTransaction
		if err := rows.Scan(
			&i.ID,
			&i.Date,
			&i.Memo,
			&i.Amount,
			&i.GroupID,
			&i.Status,
			&i.Account,
			&i.PayeeID,
			&i.CategoryID,
			&i.Payee,
			&i.CategoryGroup,
			&i.Category,
			&i.TransferAccount,
			&i.BudgetID,
			&i.AccountID,
		); err != nil {
			return nil, err
		}
		items = append(items, i)
	}
	if err := rows.Close(); err != nil {
		return nil, err
	}
	if err := rows.Err(); err != nil {
		return nil, err
	}
	return items, nil
}

const setTransactionReconciled = `-- name: SetTransactionReconciled :exec
UPDATE transactions
SET status = 'Reconciled'
WHERE id = $1
`

func (q *Queries) SetTransactionReconciled(ctx context.Context, id uuid.UUID) error {
	_, err := q.db.ExecContext(ctx, setTransactionReconciled, id)
	return err
}

const updateTransaction = `-- name: UpdateTransaction :exec
UPDATE transactions
SET date = $1,
    memo = $2,
    amount = $3,
    payee_id = $4,
    category_id = $5
WHERE id = $6
`

type UpdateTransactionParams struct {
	Date       time.Time
	Memo       string
	Amount     numeric.Numeric
	PayeeID    uuid.NullUUID
	CategoryID uuid.NullUUID
	ID         uuid.UUID
}

func (q *Queries) UpdateTransaction(ctx context.Context, arg UpdateTransactionParams) error {
	_, err := q.db.ExecContext(ctx, updateTransaction,
		arg.Date,
		arg.Memo,
		arg.Amount,
		arg.PayeeID,
		arg.CategoryID,
		arg.ID,
	)
	return err
}
