package server

import (
	"net/http"

	"github.com/gin-gonic/gin"
)

type newBudgetInformation struct {
	Name string `json:"name"`
}

func (h *Handler) newBudget(c *gin.Context) {
	var newBudget newBudgetInformation
	if err := c.BindJSON(&newBudget); err != nil {
		c.AbortWithError(http.StatusNotAcceptable, err)
		return
	}

	if newBudget.Name == "" {
		c.AbortWithStatusJSON(http.StatusBadRequest, ErrorResponse{"budget name is required"})
		return
	}

	userID := MustGetToken(c).GetID()
	budget, err := h.Service.NewBudget(c.Request.Context(), newBudget.Name, userID)
	if err != nil {
		c.AbortWithError(http.StatusInternalServerError, err)
		return
	}

	c.JSON(http.StatusOK, budget)
}
